#include "libsecurity.h"
#include <unistd.h>
#include <sys/utsname.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define LSM_PATH1	"/sys/kernel/security/lsm"
#define LSM_PATH2	"/sys/kernel/security/stacked"


#undef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
/* codes from strace package source */
static unsigned get_os_release(void)
{
        unsigned rel;
        const char *p;
        struct utsname u;
        if (uname(&u) < 0) {
                fprintf(stderr,"failed to uname: %s\n", strerror(errno));
		return -1;
	}
        /* u.release has this form: "3.2.9[-some-garbage]" */
        rel = 0;
        p = u.release;
        for (;;) {
                if (!(*p >= '0' && *p <= '9')) {
                        fprintf(stderr,"Bad OS release string: '%s'", u.release);
			return -1;
		}
                /* Note: this open-codes KERNEL_VERSION(): */
                rel = (rel << 8) | atoi(p);
                if (rel >= KERNEL_VERSION(1,0,0))
                        break;
                while (*p >= '0' && *p <= '9')
                        p++;
                if (*p != '.') {
                        if (rel >= KERNEL_VERSION(0,1,0)) {
                                /* "X.Y-something" means "X.Y.0" */
                                rel <<= 8;
                                break;
                        }
                        fprintf(stderr,"Bad OS release string: '%s'", u.release);
			return -1;
                }
                p++;
        }
        return rel;
}

int security_lsm_is_enabled(void)
{

	unsigned release =  get_os_release();

	if (release == (unsigned)-1)
		return 0;

	if (release >= KERNEL_VERSION(4, 11, 0)) {
		if (!access(LSM_PATH2, F_OK))
			return 1;
	}
	else {
		if (!access(LSM_PATH1, F_OK))
			return 1;
	}
	return 0;
}
